import { readable, derived } from 'svelte/store';
import { options } from './options';
import { getPathWithoutBase } from './utils/getPathWithoutBase';
const parseLocation = (fragment) => {
    const pathMatch = fragment.match(/^(\/[^?#]*)?/);
    const queryMatch = fragment.match(/\?([^#]*)?/);
    const hashMatch = fragment.match(/#(.*)?/);
    return {
        path: pathMatch?.[1] || '/',
        query: queryMatch?.[1] ? `?${queryMatch?.[1]}` : '',
        hash: hashMatch?.[1] ? `#${hashMatch?.[1]}` : '',
    };
};
const getWindowLocation = () => {
    const { pathname, search, hash } = document.location;
    return { path: pathname, query: search, hash: hash };
};
const getHashLocation = () => {
    let hashFragment = document.location.hash.substring(1);
    if (hashFragment[0] !== '/')
        hashFragment = '/' + hashFragment;
    return parseLocation(hashFragment);
};
const windowLocation = readable(getWindowLocation(), (set) => {
    const handler = () => set(getWindowLocation());
    window.addEventListener('popstate', handler);
    return () => window.removeEventListener('popstate', handler);
});
const hashLocation = readable(getHashLocation(), (set) => {
    const handler = () => set(getHashLocation());
    window.addEventListener('hashchange', handler);
    return () => window.removeEventListener('hashchange', handler);
});
const selectedLocation = derived([options, windowLocation, hashLocation], ([$options, $windowLocation, $hashLocation], set) => {
    if ($options.mode === 'window')
        set($windowLocation);
    if ($options.mode === 'hash')
        set($hashLocation);
});
export const path = derived(selectedLocation, ($location) => $location.path);
export const query = derived(selectedLocation, ($location) => $location.query);
export const hash = derived(selectedLocation, ($location) => $location.hash);
export const pathWithoutBase = derived([options, path], ([$options, $path]) => getPathWithoutBase($path, $options.basePath));
